# Parameters of the problem
param Nx=30;   # Number of space discretisation points
param Nt=300;  # Number of time discretisation points
# One have to check a posteriori that the number of time steps is large enough so that the
# CFL condition, T*Nx^2/Nt <=1/2, is satisfied
param dx=1/Nx; # Space step
param M =20;   # Bound on the controls

# Variables of the system
# i stands for the time index and j for the space index
var y {i in 0..Nt, j in 0..Nx} >=0; # State of the control problem
# Neuman controls in 0 and 1. The controls are in [-M,M].
var v0 {i in 0..Nt} >=-M, <=M;
var v1 {i in 0..Nt} >=-M, <=M;
var T >=0;      # Control time
var dt=T/Nt;    # Time step

# Define the cost function
minimize cost: T;

# Define the constraints
# y is solution of the discretize system
subject to y_dyn {i in 0..Nt-1, j in 1..Nx-1}:
	(y[i+1,j]-y[i,j])*(dx)^2=(y[i,j-1]-2*y[i,j]+y[i,j+1])*dt;
# Neuman boundary conditions in 0 and 1
subject to left_boundary  {i in 1..Nt-1}: y[i,1]-y[i,0]    =v0[i]*dx;
subject to right_boundary {i in 1..Nt-1}: y[i,Nx]-y[i,Nx-1]=v1[i]*dx;
# y(0)=y0 and y(T)=y1
subject to y_init {j in 0..Nx}: y[0,j] =5;
subject to y_end  {j in 0..Nx}: y[Nt,j]=1;

# Solve with IpOpt
option solver ipopt;
option ipopt_options "max_iter=2000 linear_solver=mumps halt_on_ampl_error yes";
solve;

# Write the solution in the file out.txt
printf: " # T = %24.16e\n", T >  out.txt;
printf: " # Nx = %d\n", Nx    >> out.txt;
printf: " # Nt = %d\n", Nt    >> out.txt;
printf: " # Data\n"           >> out.txt;
printf {i in 0..Nt}: " %24.16e\n", v0[i] >> out.txt;
printf {i in 0..Nt}: " %24.16e\n", v1[i] >> out.txt;
printf {i in 0..Nt, j in 0..Nx}: " %24.16e\n", y[i,j] >> out.txt;

end;
